!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Program for extracting the 300-m wind profiels dumped through ....!!
!!! GrADS from lf*0z files (height, u and v are dumped)              !!!
!!! Version: DBS (November 30, 2022)			             !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Parameter (ntime=25, nlev=180)
	Real, Dimension (nlev, ntime):: u, v, ws, wd, t, relhum, ps, w
	Real, Dimension (nlev)	     :: z
	Integer, Dimension (ntime)   :: ndd, nmm, nyyyy, nhr
	

	Open (10, file = 'DOL-300m-winds.bin', form='unformatted',      &
              access='stream')
	Open (20, file = 'DOL-300m-winds.txt', form='formatted')
	Open (3, file = 'icon_new', form='formatted')

	Read (3, 5) iyyyy,imm,idd,ihh,imn,ihrinc,itotal
 5	Format (i4, i2, i2, 4(/, i2)) 

	Do i = 1, ntime

	Call cal_jul_cal (idd,imm,iyyyy,jday,1)
	Call cal_jul_cal (31,12,iyyyy,jdaylast,1)

	jday2 = jday + 1

	iyyyy2 = iyyyy
	If (jday2.gt.jdaylast) iyyyy2 = iyyyy2 + 1
	If (jday2.gt.jdaylast) jday2 = + 1

	Call cal_jul_cal (idd2, imm2, iyyyy2, jday2, -1)

	nyyyy(i) = iyyyy2

	ihr = ihh + (i-1)*1
	nhr(i) = ihr
        If (ihr.ge.24) jday2 = jday2 + 1
        If (ihr.ge.24) nhr(i) = nhr(i) - 24
	
	If (jday2.gt.jdaylast) nyyyy(i)=nyyyy(i)+1
	If (jday2.gt.jdaylast) jday2=1

	Call cal_jul_cal (ndd(i),nmm(i),nyyyy(i),jday2,-1)

	Enddo


!	Do i1 = 10, 18,1 
	Do i1 = 10, 10,1 
	Read (i1) z
	Read (i1) u
	Read (i1) v
!	Read (i1) w
	Read (i1) t
	Read (i1) relhum
	Read (i1) ps

	Do i = 1, ntime

	mtime = (i-1)*1

	Write (i1+10, 10) ndd(i), nmm(i), nyyyy(i), nhr(i), mtime,          &
                      idd2, imm2, iyyyy2, ihh
 10	Format (/, 5x, 'COSMO Model Simulations valid for SRIHARIKOTA', &
        2(i2.2,'-'),i4.4, ': ', i2.2, ' UTC.' /,                        &
        5x, 'This is + ', i2, ' HRS forecast.' /,                       &
        5x, 'Initial conditions for this simulation is based on ',      &
        2(i2.2,'-'),i4.4, ': ', i2.2, ' UTC Data')


	Write (i1+10, 15) 'PRES(hPa)', 'ALT(m)', 'TEMP(C)', &
        'RELHUM(%)', 'WS(m/s)', 'WD(deg)', 'U(m/s)', 'V(m/s)'
	Write (i1+10, 15) '(hPa)', 'm', 'deg.C', '%', 'ms-1', 'deg',    &
        'U', 'V'
 15	Format (10 (1x, a9))

	Do k = nlev, 1, -1

	Call winds (u(k,i),v(k,i),ws(k,i),wd(k,i),1)

	If (z(k).gt.0.0) Write (i1+10, 20) ps(k,i)/100.0, z(k),  &
        t(k,i), relhum(k,i), ws(k,i), wd(k,i), u(k,i), v(k,i) 
 20	Format (8 (1x, f9.2), 2(1x, f9.4))	

	Enddo
	Write (i1+10, 21)
 21	Format (2(/))
	Enddo
	Enddo


	Stop
	
	End

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Subroutine for inter-conversion of winds ....                    !!!
!!! Forward Transformation ( U and V to WS and WD ): UV --> WS,WD    !!!
!!! Reverse Transformation ( WS and WD to U AND V ): WS,WD --> UV    !!!
!!! Version: March 25, 2010 (ATJ/DBS)                                !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                                                       
        ! u     = zonal component of winds (m/s)      
        ! v     = meridional component of winds (m/s)
        ! ws    = mean wind speed (m/s)            
        ! wd    = mean wind direction (degree)
        ! iflag = 1 for forward; -1 for reverse transformation

        Subroutine winds (u, v, ws, wd, iflag) 

        r_d = 180.0/3.14159265359       ! radian to degree conversion 
        d_r = 3.14159265359/180.0       ! degree to radian conversion 

        If (iflag == 1) Go to 201
        If (iflag == -1) Go to 202

 201    Continue

        ws = sqrt ( (u*u) + (v*v) )
        wd = ( atan2(u,v) * r_d ) + 180.0

        Go to 210

 202    Continue

        u = -1. * ws * Sin(wd) 
        v = -1. * ws * Cos(wd)

        Go to 210

 210    Return

        End
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!i!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Smbroutine for inter-conversion of Calendar and Julian Days      !!!
!!! Forward Conversion: Calendar Day to Julian Day                   !!!
!!! Reverse Conversion: Julian Day to Calendar Day                   !!!
!!! Version: April 05, 2010 (ATJ/DBS)                                !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	! idd	= date 	(Can be anything between 1 to 31)
	! imo	= month (Can be anything between 1 to 12)
	! iyr	= year 	(Can be literally anything)
	! jday	= julian day number (Can be anything between 1 to 366)
	! iflag = 1 for forward; -1 for reverse

	Subroutine cal_jul_cal (iddd,immm,iyyy,julday,iflag)
	
       	Dimension incmon(13), leapinc(13)

       	Data incmon /0, 31, 59, 90, 120, 151, 181, 212, &
     &			  243, 273, 304, 334, 365/

       	Data leapinc /0, 31, 60, 91, 121, 152, 182, 213,&
     &			  244, 274, 305, 335, 366/

       	icheck = mod (iyyy, 4) ! icheck is leap year checking index
!*		icheck = 0, if the current year is not a leap year
!*		icheck = 1, if the current year is a leap year

	If (iflag ==  1) Go to 101
	If (iflag == -1) Go to 102

 101	Continue

       	If (icheck /= 0) julday = iddd + incmon(immm)
       	If (icheck == 0) julday = iddd + leapinc(immm)

	Go to 110

 102	Continue

	If (icheck /= 0) then

       	Do i = 1, 12

       	j = i + 1

       	if ( (julday > incmon(i)).and.(julday <= incmon(j)) ) &
     &		  immm = i
       	if ( (julday > incmon(i)).and.(julday <= incmon(j)) ) & 
     &  	  iddd = julday-incmon(i)
  
       	Enddo    
        
       	Else
        
       	Do i = 1, 12

       	j = i + 1

       	if ( (julday > leapinc(i)).and.(julday <= leapinc(j)) ) &
     &		      immm = i
       	if ( (julday > leapinc(i)).and.(julday <= leapinc(j)) )  &
     &		      iddd = julday-leapinc(i)

	Enddo    

	Endif

	Go to 110

 110	Return

       	End
